@@ -89,6 +89,12 @@ char *cbmemdup(const char *ptr, int size){
 }
 
 
+/* Free a region on memory. */
+void cbfree(void *ptr){
+  free(ptr);
+}
+
+
 /* Register the pointer or handle of an object to the global garbage collector. */
 void cbglobalgc(void *ptr, void (*func)(void *)){
   assert(ptr && func);
@@ -102,6 +108,13 @@ void cbggcsweep(void){
 }
 
 
+/* Check availability of allocation of the virtual memory. */
+int cbvmemavail(size_t size){
+  assert(size >= 0);
+  return _qdbm_vmemavail(size);
+}
+
+
 /* Sort an array using insert sort. */
 void cbisort(void *base, int nmemb, int size, int(*compar)(const void *, const void *)){
   char *bp, *swap;
@@ -157,7 +170,7 @@ void cbhsort(void *base, int nmemb, int size, int(*compar)(const void *, const v
   assert(base && nmemb >= 0 && size > 0 && compar);
   bp = (char *)base;
   nmemb--;
-  bottom = nmemb / 2 +1;
+  bottom = nmemb / 2 + 1;
   top = nmemb;
   CB_MALLOC(swap, size);
   while(bottom > 0){
@@ -421,6 +434,44 @@ char *cbstrsqzspc(char *str){
 }
 
 
+/* Count the number of characters in a string of UTF-8. */
+int cbstrcountutf(const char *str){
+  const unsigned char *rp;
+  int cnt;
+  assert(str);
+  rp = (unsigned char *)str;
+  cnt = 0;
+  while(*rp != '\0'){
+    if((*rp & 0x80) == 0x00 || (*rp & 0xe0) == 0xc0 ||
+       (*rp & 0xf0) == 0xe0 || (*rp & 0xf8) == 0xf0) cnt++;
+    rp++;
+  }
+  return cnt;
+}
+
+
+/* Cut a string of UTF-8 at the specified number of characters. */
+char *cbstrcututf(char *str, int num){
+  unsigned char *wp;
+  int cnt;
+  assert(str && num >= 0);
+  wp = (unsigned char *)str;
+  cnt = 0;
+  while(*wp != '\0'){
+    if((*wp & 0x80) == 0x00 || (*wp & 0xe0) == 0xc0 ||
+       (*wp & 0xf0) == 0xe0 || (*wp & 0xf8) == 0xf0){
+      cnt++;
+      if(cnt > num){
+        *wp = '\0';
+        break;
+      }
+    }
+    wp++;
+  }
+  return str;
+}
+
+
 /* Get a datum handle. */
 CBDATUM *cbdatumopen(const char *ptr, int size){
   CBDATUM *datum;
@@ -1504,7 +1555,7 @@ CBMAP *cburlbreak(const char *str){
   assert(str);
   map = cbmapopenex(CB_MAPPBNUM);
   rp = str;
-  while(strchr(" \t\r\n\v\f", *rp)){
+  while(*rp != '\0' && strchr(" \t\r\n\v\f", *rp)){
     rp++;
   }
   tmp = cbmemdup(rp, -1);
@@ -1575,6 +1626,129 @@ CBMAP *cburlbreak(const char *str){
 }
 
 
+/* Resolve a relative URL with another absolute URL. */
+char *cburlresolve(const char *base, const char *target){
+  CBMAP *telems, *belems;
+  CBLIST *bpaths, *opaths;
+  CBDATUM *rbuf;
+  const char *vbuf, *path;
+  char *tmp;
+  int i, vsiz;
+  assert(base && target);
+  rbuf = cbdatumopen("", 0);
+  telems = cburlbreak(target);
+  if((vbuf = cbmapget(telems, "scheme", -1, &vsiz)) != NULL){
+    cbdatumcat(rbuf, vbuf, vsiz);
+    cbdatumcat(rbuf, "://", -1);
+    if((vbuf = cbmapget(telems, "authority", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, vbuf, vsiz);
+      cbdatumcat(rbuf, "@", 1);
+    }
+    if((vbuf = cbmapget(telems, "host", -1, &vsiz)) != NULL){
+      tmp = cbmemdup(vbuf, vsiz);
+      cbstrtolower(tmp);
+      cbdatumcat(rbuf, tmp, -1);
+      free(tmp);
+    } else {
+      cbdatumcat(rbuf, "localhost", -1);
+    }
+    if((vbuf = cbmapget(telems, "port", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, ":", 1);
+      cbdatumcat(rbuf, vbuf, vsiz);
+    }
+    if((vbuf = cbmapget(telems, "path", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, vbuf, vsiz);
+    } else {
+      cbdatumcat(rbuf, "/", 1);
+    }
+    if((vbuf = cbmapget(telems, "query", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, "?", 1);
+      cbdatumcat(rbuf, vbuf, vsiz);
+    }
+    if((vbuf = cbmapget(telems, "fragment", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, "#", 1);
+      cbdatumcat(rbuf, vbuf, vsiz);
+    }
+  } else {
+    belems = cburlbreak(base);
+    if((vbuf = cbmapget(belems, "scheme", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, vbuf, vsiz);
+      cbdatumcat(rbuf, "://", -1);
+    } else {
+      cbdatumcat(rbuf, "http://", -1);
+    }
+    if((vbuf = cbmapget(belems, "authority", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, vbuf, vsiz);
+      cbdatumcat(rbuf, "@", 1);
+    }
+    if((vbuf = cbmapget(belems, "host", -1, &vsiz)) != NULL){
+      tmp = cbmemdup(vbuf, vsiz);
+      cbstrtolower(tmp);
+      cbdatumcat(rbuf, tmp, -1);
+      free(tmp);
+    } else {
+      cbdatumcat(rbuf, "localhost", -1);
+    }
+    if((vbuf = cbmapget(belems, "port", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, ":", 1);
+      cbdatumcat(rbuf, vbuf, vsiz);
+    }
+    if(!(path = cbmapget(telems, "path", -1, NULL))) path = "/";
+    if(path[0] == '\0' && (vbuf = cbmapget(belems, "path", -1, &vsiz)) != NULL) path = vbuf;
+    if(path[0] == '/'){
+      cbdatumcat(rbuf, path, -1);
+    } else {
+      bpaths = cblistopen();
+      if((vbuf = cbmapget(belems, "path", -1, &vsiz)) != NULL){
+        opaths = cbsplit(vbuf, vsiz, "/");
+      } else {
+        opaths = cbsplit("/", 1, "/");
+      }
+      free(cblistpop(opaths, NULL));
+      for(i = 0; i < CB_LISTNUM(opaths); i++){
+        vbuf = CB_LISTVAL2(opaths, i, &vsiz);
+        if(vsiz < 1 || !strcmp(vbuf, ".")) continue;
+        if(!strcmp(vbuf, "..")){
+          free(cblistpop(bpaths, NULL));
+        } else {
+          cblistpush(bpaths, vbuf, vsiz);
+        }
+      }
+      cblistclose(opaths);
+      opaths = cbsplit(path, -1, "/");
+      for(i = 0; i < CB_LISTNUM(opaths); i++){
+        vbuf = CB_LISTVAL2(opaths, i, &vsiz);
+        if(vsiz < 1 || !strcmp(vbuf, ".")) continue;
+        if(!strcmp(vbuf, "..")){
+          free(cblistpop(bpaths, NULL));
+        } else {
+          cblistpush(bpaths, vbuf, vsiz);
+        }
+      }
+      cblistclose(opaths);
+      for(i = 0; i < CB_LISTNUM(bpaths); i++){
+        vbuf = CB_LISTVAL2(bpaths, i, &vsiz);
+        cbdatumcat(rbuf, "/", 1);
+        cbdatumcat(rbuf, vbuf, vsiz);
+      }
+      if(!(vbuf = cbmapget(telems, "file", -1, &vsiz)) || vsiz < 1) cbdatumcat(rbuf, "/", 1);
+      cblistclose(bpaths);
+    }
+    if((vbuf = cbmapget(telems, "query", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, "?", 1);
+      cbdatumcat(rbuf, vbuf, vsiz);
+    }
+    if((vbuf = cbmapget(telems, "fragment", -1, &vsiz)) != NULL){
+      cbdatumcat(rbuf, "#", 1);
+      cbdatumcat(rbuf, vbuf, vsiz);
+    }
+    cbmapclose(belems);
+  }
+  cbmapclose(telems);
+  return cbdatumtomalloc(rbuf, NULL);
+}
+
+
 /* Encode a serial object with URL encoding. */
 char *cburlencode(const char *ptr, int size){
   char *buf, *wp;
@@ -2406,9 +2580,16 @@ char *cbbzdecode(const char *ptr, int size, int *sp){
 
 /* Convert the character encoding of a string. */
 char *cbiconv(const char *ptr, int size, const char *icode, const char *ocode, int *sp, int *mp){
+  char *res;
   assert(ptr && icode && ocode);
   if(!_qdbm_iconv) return NULL;
-  return _qdbm_iconv(ptr, size, icode, ocode, sp, mp);
+  if((res = _qdbm_iconv(ptr, size, icode, ocode, sp, mp)) != NULL) return res;
+  if(!cbstricmp(icode, ocode)){
+    if(sp) *sp = size;
+    if(mp) *mp = 0;
+    return cbmemdup(ptr, size);
+  }
+  return NULL;
 }
 
 
