@@ -100,7 +100,7 @@ void usage(void){
   fprintf(stderr, "%s: popular encoders and decoders\n", progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s url [-d] [-br] [-l] [-e expr] [file]\n", progname);
+  fprintf(stderr, "  %s url [-d] [-br] [-rs base target] [-l] [-e expr] [file]\n", progname);
   fprintf(stderr, "  %s base [-d] [-l] [-c num] [-e expr] [file]\n", progname);
   fprintf(stderr, "  %s quote [-d] [-l] [-c num] [-e expr] [file]\n", progname);
   fprintf(stderr, "  %s mime [-d] [-hd] [-bd] [-part num] [-l] [-ec code] [-qp] [-dc] [-e expr]"
@@ -120,7 +120,7 @@ void usage(void){
     free(tmp);
   }
   if((tmp = cbiconv("", 0, "US-ASCII", "US-ASCII", NULL, NULL)) != NULL){
-    fprintf(stderr, "  %s iconv [-ic code] [-oc code] [-ol ltype] [-cn] [-um] [file]\n",
+    fprintf(stderr, "  %s iconv [-ic code] [-oc code] [-ol ltype] [-cn] [-wc] [-um] [file]\n",
             progname);
     free(tmp);
   }
@@ -151,10 +151,12 @@ int runurl(int argc, char **argv){
   CBMAP *map;
   int i, size, dec, br, line;
   const char *val;
-  char *expr, *file, *buf, *res;
+  char *base, *target, *expr, *file, *buf, *res;
   dec = FALSE;
   br = FALSE;
   line = FALSE;
+  base = NULL;
+  target = NULL;
   expr = NULL;
   file = NULL;
   for(i = 2; i < argc; i++){
@@ -163,6 +165,11 @@ int runurl(int argc, char **argv){
         dec = TRUE;
       } else if(!strcmp(argv[i], "-br")){
         br = TRUE;
+      } else if(!strcmp(argv[i], "-rs")){
+        if(++i >= argc) usage();
+        base = argv[i];
+        if(++i >= argc) usage();
+        target = argv[i];
       } else if(!strcmp(argv[i], "-l")){
         line = TRUE;
       } else if(!strcmp(argv[i], "-e")){
@@ -178,7 +185,10 @@ int runurl(int argc, char **argv){
     }
   }
   buf = NULL;
-  if(expr){
+  if(base){
+    size = strlen(base);
+    buf = cbmemdup(base, size);
+  } else if(expr){
     size = strlen(expr);
     buf = cbmemdup(expr, size);
   } else if(file){
@@ -189,7 +199,11 @@ int runurl(int argc, char **argv){
   } else {
     buf = readstdin(&size);
   }
-  if(br){
+  if(target){
+    res = cburlresolve(base, target);
+    printf("%s", res);
+    free(res);
+  } else if(br){
     map = cburlbreak(buf);
     if((val = cbmapget(map, "self", -1, NULL))) printf("self\t%s\n", val);
     if((val = cbmapget(map, "scheme", -1, NULL))) printf("scheme\t%s\n", val);
@@ -867,12 +881,13 @@ int runiconv(int argc, char **argv){
   CBDATUM *datum;
   const char *rcode;
   char *icode, *ocode, *ltype, *file, *buf, *res, *norm, *orig;
-  int i, cn, bsiz, rsiz, nsiz, osiz, miss;
+  int i, cn, wc, bsiz, rsiz, nsiz, osiz, miss;
   icode = NULL;
   ocode = NULL;
   ltype = NULL;
   file = NULL;
   cn = FALSE;
+  wc = FALSE;
   for(i = 2; i < argc; i++){
     if(!file && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-ic")){
@@ -886,6 +901,8 @@ int runiconv(int argc, char **argv){
         ltype = argv[i];
       } else if(!strcmp(argv[i], "-cn")){
         cn = TRUE;
+      } else if(!strcmp(argv[i], "-wc")){
+        wc = TRUE;
       } else if(!strcmp(argv[i], "-um")){
         shouucsmap();
       } else {
@@ -906,8 +923,11 @@ int runiconv(int argc, char **argv){
   } else {
     buf = readstdin(&bsiz);
   }
+  miss = 0;
   if(cn){
     printf("%s\n", cbencname(buf, bsiz));
+  } else if(wc){
+    printf("%d\n", cbstrcountutf(buf));
   } else {
     rcode = icode ? icode : cbencname(buf, bsiz);
     if(!(res = cbiconv(buf, bsiz, rcode, ocode ? ocode : DEFCODE,
