@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Utility for debugging Curia and its applications
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -108,7 +108,7 @@ int main(int argc, char **argv){
   } else if(!strcmp(argv[1], "version") || !strcmp(argv[1], "--version")){
     printf("Powered by QDBM version %s on %s%s\n",
            dpversion, dpsysname, dpisreentrant ? " (reentrant)" : "");
-    printf("Copyright (c) 2000-2005 Mikio Hirabayashi\n");
+    printf("Copyright (c) 2000-2006 Mikio Hirabayashi\n");
     rv = 0;
   } else {
     usage();
@@ -794,7 +794,7 @@ int doinform(const char *name){
   printf("used buckets: %d\n", crbusenum(curia));
   printf("records: %d\n", crrnum(curia));
   printf("inode number: %d\n", crinode(curia));
-  printf("modified time: %d\n", crmtime(curia));
+  printf("modified time: %.0f\n", (double)crmtime(curia));
   printf("LOB records: %d\n", crrnumlob(curia));
   if(!crclose(curia)){
     pdperror(name);
