@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Curia
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -14,6 +14,8 @@
  *************************************************************************************************/
 
 
+#define QDBM_INTERNAL  1
+
 #include "curia.h"
 #include "myconf.h"
 
@@ -314,6 +316,25 @@ int crsetalign(CURIA *curia, int align){
 }
 
 
+/* Set the size of the free block pool of a database handle. */
+int crsetfbpsiz(CURIA *curia, int size){
+  int i, err;
+  assert(curia && size >= 0);
+  if(!curia->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  for(i = 0; i < curia->dnum; i++){
+    if(!dpsetfbpsiz(curia->depots[i], size)){
+      err = TRUE;
+      break;
+    }
+  }
+  return err ? FALSE : TRUE;
+}
+
+
 /* Synchronize contents of updating a database with the files and the devices. */
 int crsync(CURIA *curia){
   int i, err;
@@ -465,7 +486,7 @@ int crinode(CURIA *curia){
 
 
 /* Get the last modified time of a database. */
-int crmtime(CURIA *curia){
+time_t crmtime(CURIA *curia){
   assert(curia);
   return dpmtime(curia->attr);
 }
