@@ -33,6 +33,7 @@
 #define DP_ENTBUFSIZ   128               /* size of the entity buffer */
 #define DP_STKBUFSIZ   256               /* size of the stack key buffer */
 #define DP_WRTBUFSIZ   8192              /* size of the writing buffer */
+#define DP_FSBLKSIZ    4096              /* size of a block of the file system */
 #define DP_TMPFSUF     MYEXTSTR "dptmp"  /* suffix of a temporary file */
 #define DP_OPTBLOAD    0.25              /* ratio of bucket loading at optimization */
 #define DP_OPTRUNIT    256               /* number of records in a process of optimization */
@@ -70,7 +71,7 @@ static int dpfirsthash(const char *kbuf, int ksiz);
 static int dpsecondhash(const char *kbuf, int ksiz);
 static int dpthirdhash(const char *kbuf, int ksiz);
 static int dpgetprime(int num);
-static int dppadsize(DEPOT *depot, int vsiz);
+static int dppadsize(DEPOT *depot, int ksiz, int vsiz);
 static int dprecsize(int *head);
 static int dprechead(DEPOT *depot, int off, int *head, char *ebuf, int *eep);
 static char *dpreckey(DEPOT *depot, int off, int *head);
@@ -1621,15 +1622,25 @@ static int dpgetprime(int num){
 /* Get the padding size of a record.
    `vsiz' specifies the size of the value of a record.
    The return value is the padding size of a record. */
-static int dppadsize(DEPOT *depot, int vsiz){
-  int mod, pad;
+static int dppadsize(DEPOT *depot, int ksiz, int vsiz){
+  int pad;
   assert(depot && vsiz >= 0);
   if(depot->align > 0){
-    mod = vsiz % depot->align;
-    return mod == 0 ? 0 : depot->align - mod;
+    return depot->align - (depot->fsiz + DP_RHNUM * sizeof(int) + ksiz + vsiz) % depot->align;
   } else if(depot->align < 0){
     pad = (int)(vsiz * (2.0 / (1 << -(depot->align))));
-    return pad >= DP_RHNUM * sizeof(int) ? pad : DP_RHNUM * sizeof(int);
+    if(vsiz + pad >= DP_FSBLKSIZ){
+      if(vsiz <= DP_FSBLKSIZ) pad = 0;
+      if(depot->fsiz % DP_FSBLKSIZ == 0){
+        return (pad / DP_FSBLKSIZ) * DP_FSBLKSIZ + DP_FSBLKSIZ -
+          (depot->fsiz + DP_RHNUM * sizeof(int) + ksiz + vsiz) % DP_FSBLKSIZ;
+      } else {
+        return (pad / (DP_FSBLKSIZ / 2)) * (DP_FSBLKSIZ / 2) + (DP_FSBLKSIZ / 2) -
+          (depot->fsiz + DP_RHNUM * sizeof(int) + ksiz + vsiz) % (DP_FSBLKSIZ / 2);
+      }
+    } else {
+      return pad >= DP_RHNUM * sizeof(int) ? pad : DP_RHNUM * sizeof(int);
+    }
   }
   return 0;
 }
@@ -1897,7 +1908,7 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
   char ebuf[DP_WRTBUFSIZ], *hbuf;
   int head[DP_RHNUM], asiz, psiz, off;
   assert(depot && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
-  psiz = dppadsize(depot, vsiz);
+  psiz = dppadsize(depot, ksiz, vsiz);
   head[DP_RHIFLAGS] = 0;
   head[DP_RHIHASH] = hash;
   head[DP_RHIKSIZ] = ksiz;
