@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Java API of Curia, the extended API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -316,7 +316,7 @@ public class Curia implements ADBM {
    * Set alignment of the database handle.
    * @param align the basic size of alignment.
    * @throws CuriaException if an error occurs.
-   * @note If alignment is set to a database, the efficiency of overwriting values are improved.
+   * @note If alignment is set to a database, the efficiency of overwriting values is improved.
    * The size of alignment is suggested to be average size of the values of the records to be
    * stored.  If alignment is positive, padding whose size is multiple number of the alignment
    * is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -329,6 +329,19 @@ public class Curia implements ADBM {
       if(crsetalign(index, align) == 0) throw new CuriaException(crecode());
     }
   }
+  /**
+   * Set the size of the free block pool.
+   * @param size the size of the free block pool of a database.
+   * @throws CuriaException if an error occurs.
+   * @note The default size of the free block pool is 16.  If the size is greater, the space
+   * efficiency of overwriting values is improved with the time efficiency sacrificed.
+   */
+  public void setfbpsiz(int size) throws CuriaException {
+    if(index < 0) throw new CuriaException();
+    synchronized(ADBM.class){
+      if(crsetfbpsiz(index, size) == 0) throw new CuriaException(crecode());
+    }
+  }
   /**
    * Synchronize updating contents with the files and the devices.
    * @throws CuriaException if an error occurs.
@@ -442,6 +455,28 @@ public class Curia implements ADBM {
       return crfatalerror(index) == 0 ? false : true;
     }
   }
+  /**
+   * Get the inode number of the database.
+   * @return the inode number of the database directory.
+   * @throws CuriaException if an error occurs.
+   */
+  public int inode() throws CuriaException {
+    if(index < 0) throw new CuriaException();
+    synchronized(ADBM.class){
+      return crinode(index);
+    }
+  }
+  /**
+   * Get the last modified time of the database.
+   * @return the last modified time of the database.
+   * @throws CuriaException if an error occurs.
+   */
+  public long mtime() throws CuriaException {
+    if(index < 0) throw new CuriaException();
+    synchronized(ADBM.class){
+      return crmtime(index);
+    }
+  }
   /**
    * Store a large object.
    * @param key a byte array of a key.
@@ -602,6 +637,7 @@ public class Curia implements ADBM {
   private static synchronized final native int criterinit(int index);
   private static synchronized final native byte[] criternext(int index);
   private static synchronized final native int crsetalign(int index, int align);
+  private static synchronized final native int crsetfbpsiz(int index, int size);
   private static synchronized final native int crsync(int index);
   private static synchronized final native int croptimize(int index, int bnum);
   private static synchronized final native String crname(int index);
@@ -611,6 +647,8 @@ public class Curia implements ADBM {
   private static synchronized final native int crrnum(int index);
   private static synchronized final native int crwritable(int index);
   private static synchronized final native int crfatalerror(int index);
+  private static synchronized final native int crinode(int index);
+  private static synchronized final native long crmtime(int index);
   private static synchronized final native int crputlob(int index, byte[] key, int ksiz,
                                                      byte[] val, int vsiz, int dmode);
   private static synchronized final native int croutlob(int index, byte[] key, int ksiz);
