@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Java API of Villa, the advanced API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -83,8 +83,12 @@ public class Villa implements ADBM {
   public static final int ONOLCK = 1 << 4;
   /** open mode: lock without blocking */
   public static final int OLCKNB = 1 << 5;
-  /** open mode: compress leaves */
+  /** open mode: compress leaves with ZLIB */
   public static final int OZCOMP = 1 << 6;
+  /** open mode: compress leaves with LZO */
+  public static final int OYCOMP = 1 << 7;
+  /** open mode: compress leaves with BZIP2 */
+  public static final int OXCOMP = 1 << 8;
   //----------------------------------------------------------------
   // comparing modes
   //----------------------------------------------------------------
@@ -107,6 +111,8 @@ public class Villa implements ADBM {
   public static final int DCAT = 2;
   /** write mode: allow duplication of records */
   public static final int DDUP = 3;
+  /** write mode: allow duplication with reverse order */
+  public static final int DDUPR = 4;
   //----------------------------------------------------------------
   // jump modes
   //----------------------------------------------------------------
@@ -114,6 +120,12 @@ public class Villa implements ADBM {
   public static final int JFORWARD = 0;
   /** jump mode: jump mode: step backward */
   public static final int JBACKWARD = 1;
+  /** insertion mode: overwrite the current record */
+  public static final int CPCURRENT = 0;
+  /** insertion mode: insert before the current record */
+  public static final int CPBEFORE = 1;
+  /** insertion mode: insert after the current record */
+  public static final int CPAFTER = 2;
   //----------------------------------------------------------------
   // static initializer
   //----------------------------------------------------------------
@@ -172,9 +184,11 @@ public class Villa implements ADBM {
    * a reader.  If the mode is `Villa.OWRITER', the following may be added by bitwise or:
    * `Villa.OCREAT', which means it creates a new database if not exist, `Villa.OTRUNC',
    * which means it creates a new database regardless if one exists, `Villa.OZCOMP', which means
-   * leaves in the database are compressed.  Both of `Villa.OREADER' and `Villa.OWRITER' can be
-   * added to by bitwise or: `Villa.ONOLCK', which means it opens a database file without file
-   * locking, or `Villa.OLCKNB', which means locking is performed without blocking.
+   * leaves in the database are compressed with ZLIB, `Villa.OYCOMP', which means leaves in the
+   * database are compressed with LZO, `Villa.OXCOMP', which means leaves in the database are
+   * compressed with BZIP2.  Both of `Villa.OREADER' and `Villa.OWRITER' can be added to by
+   * bitwise or: `Villa.ONOLCK', which means it opens a database file without file locking, or
+   * `Villa.OLCKNB', which means locking is performed without blocking.
    * @param cmode the comparing function: `Villa.CMPLEX' comparing keys in lexical order,
    * `Villa.CMPNUM' comparing keys as numbers of big endian, `Villa.CMPDEC' comparing keys as
    * decimal strings, `Villa.CMPOBJ' comparing keys as serialized objects implementing
@@ -182,9 +196,9 @@ public class Villa implements ADBM {
    * database.
    * @note While connecting as a writer, an exclusive lock is invoked to the database file.
    * While connecting as a reader, a shared lock is invoked to the database file.  The thread
-   * blocks until the lock is achieved.  `Villa.OZCOMP' is available only if QDBM was built with
-   * ZLIB enabled.  If `Villa.ONOLCK' is used, the application is responsible for exclusion
-   * control.
+   * blocks until the lock is achieved.  `Villa.OZCOMP', `Villa.OYCOMP', and `Villa.OXCOMP' are
+   * available only if QDBM was built each with ZLIB, LZO, and BZIP2 enabled.  If `Villa.ONOLCK'
+   * is used, the application is responsible for exclusion control.
    */
   public Villa(String name, int omode, int cmode) throws VillaException {
     synchronized(ADBM.class){
@@ -237,12 +251,13 @@ public class Villa implements ADBM {
    * @param key a byte array of a key.
    * @param val a byte array of a value.
    * @param dmode behavior when the key overlaps, by the following values: `Villa.DOVER',
-   * which means the specified value overwrites the existing one, `Villa.DKEEP', which means the
-   * existing value is kept, `Villa.DCAT', which means the specified value is concatenated at the
-   * end of the existing value, `Villa.DDUP', which means duplication of keys is allowed.
+   * which means the specified value overwrites the existing one, `Villa.DKEEP', which means
+   * the existing value is kept, `Villa.DCAT', which means the specified value is concatenated
+   * at the end of the existing value, `Villa.DDUP', which means duplication of keys is allowed
+   * and the specified value is added as the last one, `Villa.DDUPR', which means duplication of
+   * keys is allowed and the specified value is added as the first one.
    * @throws VillaException if an error occurs or replace is cancelled.
-   * @note A duplicated record is stored at the tail of the records of the same key.  The cursor
-   * becomes unavailable due to updating database.
+   * @note The cursor becomes unavailable due to updating database.
    */
   public void put(byte[] key, byte[] val, int dmode) throws VillaException {
     if(index < 0) throw new VillaException();
@@ -496,6 +511,51 @@ public class Villa implements ADBM {
     if(val == null) throw new VillaException();
     return val;
   }
+  /**
+   * Insert a record around the cursor.
+   * @param val a byte array of a value.
+   * @param cpmode detail adjustment: `Villa.CPCURRENT', which means that the value of the
+   * current record is overwritten, `Villa.CPBEFORE', which means that a new record is inserted
+   * before the current record, `Villa.CPAFTER', which means that a new record is inserted after
+   * the current record.
+   * @throws VillaException if an error occurs or no record corresponds to the cursor.
+   * @note After insertion, the cursor is moved to the inserted record.
+   */
+  public void curput(byte[] val, int cpmode) throws VillaException {
+    if(index < 0) throw new VillaException();
+    synchronized(ADBM.class){
+      if(vlcurput(index, val, val.length, cpmode) == 0) throw new VillaException(vlecode());
+    }
+  }
+  /**
+   * Insert a record as the successor of the cursor.
+   * The same as `curput(val, Villa.CPCURRENT)'.
+   * @see #curput(byte[], int)
+   */
+  public void curput(byte[] val) throws VillaException {
+    curput(val, CPCURRENT);
+  }
+  /**
+   * Insert a record around the cursor.
+   * The same as `curput(qdbm.Util.serialize(val), cpmode)'.
+   * @see #curput(byte[], int)
+   */
+  public void curputobj(Object val, int cpmode) throws VillaException {
+    byte[] vbuf = Util.serialize(val);
+    if(vbuf == null) throw new VillaException();
+    curput(vbuf, cpmode);
+  }
+  /**
+   * Delete the record where the cursor is.
+   * @throws VillaException if an error occurs or no record corresponds to the cursor.
+   * @note After deletion, the cursor is moved to the next record if possible.
+   */
+  public void curout() throws VillaException {
+    if(index < 0) throw new VillaException();
+    synchronized(ADBM.class){
+      if(vlcurout(index) == 0) throw new VillaException(vlecode());
+    }
+  }
   /**
    * Set the tuning parameters for performance.
    * @param lrecmax the max number of records in a leaf node of B+ tree.  If it is not more
@@ -629,6 +689,28 @@ public class Villa implements ADBM {
       return vlfatalerror(index) == 0 ? false : true;
     }
   }
+  /**
+   * Get the inode number of the database.
+   * @return the inode number of the database file.
+   * @throws VillaException if an error occurs.
+   */
+  public int inode() throws VillaException {
+    if(index < 0) throw new VillaException();
+    synchronized(ADBM.class){
+      return vlinode(index);
+    }
+  }
+  /**
+   * Get the last modified time of the database.
+   * @return the last modified time of the database.
+   * @throws VillaException if an error occurs.
+   */
+  public long mtime() throws VillaException {
+    if(index < 0) throw new VillaException();
+    synchronized(ADBM.class){
+      return vlmtime(index);
+    }
+  }
   /**
    * Begin the transaction.
    * @throws VillaException if an error occurs.
@@ -800,6 +882,9 @@ public class Villa implements ADBM {
                                                          int jmode);
   private static synchronized final native byte[] vlcurkey(int index);
   private static synchronized final native byte[] vlcurval(int index);
+  private static synchronized final native int vlcurput(int index, byte[] val, int vsiz,
+                                                        int cpmode);
+  private static synchronized final native int vlcurout(int index);
   private static synchronized final native void vlsettuning(int index, int lrecmax, int nidxmax,
                                                             int lcnum, int ncnum);
   private static synchronized final native int vlsync(int index);
@@ -811,6 +896,8 @@ public class Villa implements ADBM {
   private static synchronized final native int vlrnum(int index);
   private static synchronized final native int vlwritable(int index);
   private static synchronized final native int vlfatalerror(int index);
+  private static synchronized final native int vlinode(int index);
+  private static synchronized final native long vlmtime(int index);
   private static synchronized final native int vltranbegin(int index);
   private static synchronized final native int vltrancommit(int index);
   private static synchronized final native int vltranabort(int index);
