@@ -62,8 +62,8 @@ int _qdbm_win32_lstat(const char *pathname, struct stat *buf){
 #define PTKEYMAX       8
 
 
-static struct { void *ptr; pthread_key_t key; } _qdbm_ptkeys[PTKEYMAX];
-static int _qdbm_ptknum = 0;
+struct { void *ptr; pthread_key_t key; } _qdbm_ptkeys[PTKEYMAX];
+int _qdbm_ptknum = 0;
 
 
 static void *_qdbm_gettsd(void *ptr, int size, const void *initval);
@@ -149,12 +149,17 @@ static void _qdbm_delete_mmap_env(void);
 
 
 void *_qdbm_mmap(void *start, size_t length, int prot, int flags, int fd, off_t offset){
-  static int first = FALSE;
+  static volatile long first = TRUE;
+  static volatile long ready = FALSE;
   HANDLE handle;
   int i;
-  if(!InterlockedCompareExchange(&first, TRUE, FALSE)){
+  if(InterlockedExchange((long *)&first, FALSE)){
     InitializeCriticalSection(&mmcsec);
     atexit(_qdbm_delete_mmap_env);
+    InterlockedExchange((long *)&ready, TRUE);
+  }
+  while(!InterlockedCompareExchange((long *)&ready, TRUE, TRUE)){
+    Sleep(1);
   }
   if(fd < 0 || flags & MAP_FIXED) return MAP_FAILED;
   if(!(handle = CreateFileMapping((HANDLE)_get_osfhandle(fd), NULL,
@@ -465,6 +470,61 @@ struct dirent *_qdbm_win32_readdir(DIR *dir){
 
 
 
+/*************************************************************************************************
+ * for checking information of the system
+ *************************************************************************************************/
+
+
+#if defined(_SYS_LINUX_)
+
+
+int _qdbm_vmemavail(size_t size){
+  char buf[4096], *rp;
+  int fd, rv, bsiz;
+  double avail;
+  if((fd = open("/proc/meminfo", O_RDONLY, 00644)) == -1) return TRUE;
+  rv = TRUE;
+  if((bsiz = read(fd, buf, sizeof(buf) - 1)) > 0){
+    buf[bsiz] = '\0';
+    avail = -1;
+    if((rp = strstr(buf, "MemFree:")) != NULL){
+      rp = strchr(rp, ':') + 1;
+      avail = strtod(rp, NULL) * 1024.0;
+      if((rp = strstr(buf, "SwapFree:")) != NULL){
+        rp = strchr(rp, ':') + 1;
+        avail += strtod(rp, NULL) * 1024.0;
+      }
+      if(size >= avail) rv = FALSE;
+    }
+  }
+  close(fd);
+  return rv;
+}
+
+
+#elif defined(_SYS_MSVC_) || defined(_SYS_MINGW_) || defined(_SYS_CYGWIN_)
+
+
+int _qdbm_vmemavail(size_t size){
+  MEMORYSTATUS sbuf;
+  sbuf.dwLength = sizeof(MEMORYSTATUS);
+  GlobalMemoryStatus(&sbuf);
+  return size < sbuf.dwAvailVirtual;
+}
+
+
+#else
+
+
+int _qdbm_vmemavail(size_t size){
+  return TRUE;
+}
+
+
+#endif
+
+
+
 /*************************************************************************************************
  * for ZLIB
  *************************************************************************************************/
