@@ -482,6 +482,15 @@ struct dirent *_qdbm_win32_readdir(DIR *dir);
 
 
 
+/*************************************************************************************************
+ * for checking information of the system
+ *************************************************************************************************/
+
+
+int _qdbm_vmemavail(size_t size);
+
+
+
 /*************************************************************************************************
  * for ZLIB
  *************************************************************************************************/
