@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Odeum
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -14,6 +14,8 @@
  *************************************************************************************************/
 
 
+#define QDBM_INTERNAL  1
+
 #include "odeum.h"
 #include "myconf.h"
 
@@ -28,9 +30,11 @@
 #define OD_DOCSBNUM    2039              /* initial bucket number of document database */
 #define OD_DOCSDNUM    17                /* division number of document database */
 #define OD_DOCSALIGN   -4                /* alignment of document database */
+#define OD_DOCSFBP     32                /* size of free block pool of document database */
 #define OD_INDEXBNUM   32749             /* initial bucket number of inverted index */
 #define OD_INDEXDNUM   7                 /* division number of inverted index */
 #define OD_INDEXALIGN  -2                /* alignment of inverted index */
+#define OD_INDEXFBP    32                /* size of free block pool of inverted index */
 #define OD_RDOCSLRM    81                /* records in a leaf node of reverse dictionary */
 #define OD_RDOCSNIM    192               /* records in a non-leaf node of reverse dictionary */
 #define OD_RDOCSLCN    128               /* number of leaf cache of reverse dictionary */
@@ -754,7 +758,7 @@ int odinode(ODEUM *odeum){
 
 
 /* Get the last modified time of a database. */
-int odmtime(ODEUM *odeum){
+time_t odmtime(ODEUM *odeum){
   assert(odeum);
   return crmtime(odeum->indexdb);
 }
@@ -1547,7 +1551,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
   vlomode = VL_OREADER;
   if(omode & OD_OWRITER){
     cromode = CR_OWRITER;
-    vlomode = VL_OWRITER | VL_OZCOMP;
+    vlomode = VL_OWRITER | VL_OZCOMP | VL_OYCOMP;
     if(omode & OD_OCREAT){
       cromode |= CR_OCREAT;
       vlomode |= VL_OCREAT;
@@ -1588,7 +1592,8 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
     return NULL;
   }
   if(omode & OD_OWRITER){
-    if(!crsetalign(docsdb, OD_DOCSALIGN) || !crsetalign(indexdb, OD_INDEXALIGN)){
+    if(!crsetalign(docsdb, OD_DOCSALIGN) || !crsetfbpsiz(docsdb, OD_DOCSFBP) ||
+       !crsetalign(indexdb, OD_INDEXALIGN) || !crsetfbpsiz(indexdb, OD_INDEXFBP)){
       crclose(indexdb);
       crclose(docsdb);
       return NULL;
