@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * C++ API of Curia, the extended API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -25,6 +25,7 @@ extern "C" {
 #include <depot.h>
 #include <curia.h>
 #include <stdlib.h>
+#include <time.h>
 }
 
 
@@ -307,7 +308,7 @@ public:
    * Set alignment of the database handle.
    * @param align the basic size of alignment.
    * @throw Curia_error if an error occurs.
-   * @note If alignment is set to a database, the efficiency of overwriting values are improved.
+   * @note If alignment is set to a database, the efficiency of overwriting values is improved.
    * The size of alignment is suggested to be average size of the values of the records to be
    * stored.  If alignment is positive, padding whose size is multiple number of the alignment
    * is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -315,6 +316,14 @@ public:
    * saved in a database, you should specify alignment every opening a database.
    */
   virtual void setalign(int align) throw(Curia_error);
+  /**
+   * Set the size of the free block pool.
+   * @param size the size of the free block pool of a database.
+   * @throw Curia_error if an error occurs.
+   * @note The default size of the free block pool is 16.  If the size is greater, the space
+   * efficiency of overwriting values is improved with the time efficiency sacrificed.
+   */
+  virtual void setfbpsiz(int size) throw(Curia_error);
   /**
    * Synchronize updating contents with the files and the devices.
    * @note Curia_error if an error occurs.
@@ -393,7 +402,7 @@ public:
    * @return the last modified time the database.
    * @throw Curia_error if an error occurs.
    */
-  virtual int mtime() throw(Curia_error);
+  virtual time_t mtime() throw(Curia_error);
   /**
    * Store a large object.
    * @param kbuf the pointer to the region of a key.
