@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * C++ API of Depot, the basic API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -24,6 +24,7 @@
 extern "C" {
 #include <depot.h>
 #include <stdlib.h>
+#include <time.h>
 }
 
 
@@ -303,7 +304,7 @@ public:
    * Set alignment of the database handle.
    * @param align the basic size of alignment.
    * @throw Depot_error if an error occurs.
-   * @note If alignment is set to a database, the efficiency of overwriting values are improved.
+   * @note If alignment is set to a database, the efficiency of overwriting values is improved.
    * The size of alignment is suggested to be average size of the values of the records to be
    * stored.  If alignment is positive, padding whose size is multiple number of the alignment
    * is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -311,6 +312,14 @@ public:
    * saved in a database, you should specify alignment every opening a database.
    */
   virtual void setalign(int align) throw(Depot_error);
+  /**
+   * Set the size of the free block pool.
+   * @param size the size of the free block pool of a database.
+   * @throw Depot_error if an error occurs.
+   * @note The default size of the free block pool is 16.  If the size is greater, the space
+   * efficiency of overwriting values is improved with the time efficiency sacrificed.
+   */
+  virtual void setfbpsiz(int size) throw(Depot_error);
   /**
    * Synchronize updating contents with the file and the device.
    * @throw Depot_error if an error occurs.
@@ -382,7 +391,7 @@ public:
    * @return the last modified time the database.
    * @throw Depot_error if an error occurs.
    */
-  virtual int mtime() throw(Depot_error);
+  virtual time_t mtime() throw(Depot_error);
   /**
    * Get the file descriptor of the database file.
    * @return the file descriptor of the database file.
