@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * C++ API of Villa, the advanced API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -26,6 +26,7 @@ extern "C" {
 #include <cabin.h>
 #include <villa.h>
 #include <stdlib.h>
+#include <time.h>
 }
 
 
@@ -152,13 +153,19 @@ public:
   static const int OTRUNC;            ///< open mode: writer truncating
   static const int ONOLCK;            ///< open mode: open without locking
   static const int OLCKNB;            ///< open mode: lock without blocking
-  static const int OZCOMP;            ///< open mode: compress leaves
+  static const int OZCOMP;            ///< open mode: compress leaves with ZLIB
+  static const int OYCOMP;            ///< open mode: compress leaves with LZO
+  static const int OXCOMP;            ///< open mode: compress leaves with BZIP2
   static const int DOVER;             ///< write mode: overwrite the existing value
   static const int DKEEP;             ///< write mode: keep the existing value
   static const int DCAT;              ///< write mode: concatenate values
-  static const int DDUP;              ///< write mode: allow duplication of records
+  static const int DDUP;              ///< write mode: allow duplication of keys
+  static const int DDUPR;             ///< write mode: allow duplication with reverse order
   static const int JFORWARD;          ///< jump mode: step forward
   static const int JBACKWARD;         ///< jump mode: step backward
+  static const int CPCURRENT;         ///< insertion mode: overwrite the current record
+  static const int CPBEFORE;          ///< insertion mode: insert before the current record
+  static const int CPAFTER;           ///< insertion mode: insert after the current record
   //----------------------------------------------------------------
   // public static member functions
   //----------------------------------------------------------------
@@ -225,9 +232,11 @@ public:
    * a reader.  If the mode is `Villa::OWRITER', the following may be added by bitwise or:
    * `Villa::OCREAT', which means it creates a new database if not exist, `Villa::OTRUNC',
    * which means it creates a new database regardless if one exists, `Villa::OZCOMP', which means
-   * leaves in the database are compressed.  Both of `Villa::OREADER' and `Villa::OWRITER' can
-   * be added to by bitwise or: `Villa::ONOLCK', which means it opens a database file without
-   * file locking, or `Villa::OLCKNB', which means locking is performed without blocking.
+   * leaves in the database are compressed with ZLIB, `Villa::OYCOMP', which means leaves in the
+   * database are compressed with LZO, `Villa::OXCOMP', which means leaves in the database are
+   * compressed with BZIP2.  Both of `Villa::OREADER' and `Villa::OWRITER' can be added to by
+   * bitwise or: `Villa::ONOLCK', which means it opens a database file without file locking, or
+   * `Villa::OLCKNB', which means locking is performed without blocking.
    * @param cmp the comparing function: `Villa::cmplex' comparing keys in lexical order,
    * `Villa::cmpint' comparing keys as objects of `int' in native byte order, `Villa::cmpnum'
    * comparing keys as numbers of big endian, `Villa::cmpdec' comparing keys as decimal strings.
@@ -236,9 +245,9 @@ public:
    * @throw Villa_error if an error occurs.
    * @note While connecting as a writer, an exclusive lock is invoked to the database file.
    * While connecting as a reader, a shared lock is invoked to the database file.  The thread
-   * blocks until the lock is achieved.  `Villa::OZCOMP' is available only if QDBM was built with
-   * ZLIB enabled.  If `Villa::ONOLCK' is used, the application is responsible for exclusion
-   * control.
+   * blocks until the lock is achieved.  `Villa::OZCOMP', `Villa::OYCOMP', and `Villa::OXCOMP'
+   * are available only if QDBM was built each with ZLIB, LZO, and BZIP2 enabled.  If
+   * `Villa::ONOLCK' is used, the application is responsible for exclusion control.
    */
   Villa(const char* name, int omode = Villa::OREADER, VLCFUNC cmp = Villa::cmplex)
     throw(Villa_error);
@@ -266,10 +275,11 @@ public:
    * @param dmode behavior when the key overlaps, by the following values: `Villa::DOVER',
    * which means the specified value overwrites the existing one, `Villa::DKEEP', which means
    * the existing value is kept, `Villa::DCAT', which means the specified value is concatenated
-   * at the end of the existing value, `Villa::DDUP', which means duplication of keys is allowed.
+   * at the end of the existing value, `Villa::DDUP', which means duplication of keys is allowed
+   * and the specified value is added as the last one, `Villa::DDUPR', which means duplication of
+   * keys is allowed and the specified value is added as the first one.
    * @throw Villa_error if an error occurs or replace is cancelled.
-   * @note A duplicated record is stored at the tail of the records of the same key.  The
-   * cursor becomes unavailable due to updating database.
+   * @note The cursor becomes unavailable due to updating database.
    */
   virtual void put(const char* kbuf, int ksiz, const char* vbuf, int vsiz,
                    int dmode = Villa::DOVER) throw(Villa_error);
@@ -376,6 +386,26 @@ public:
    * `std::free' call if it is no longer in use.
    */
   virtual char* curval(int* sp = 0) throw(Villa_error);
+  /**
+   * Insert a record around the cursor.
+   * @param vbuf the pointer to the region of a value.
+   * @param vsiz the size of the region of the value.  If it is negative, the size is assigned
+   * with `std::strlen(vbuf)'.
+   * @param cpmode detail adjustment: `Villa::CPCURRENT', which means that the value of the
+   * current record is overwritten, `Villa::CPBEFORE', which means that a new record is inserted
+   * before the current record, `Villa::CPAFTER', which means that a new record is inserted after
+   * the current record.
+   * @throw Villa_error if an error occurs or no record corresponds to the cursor.
+   * @note After insertion, the cursor is moved to the inserted record.
+   */
+  virtual void curput(const char* vbuf, int vsiz, int cpmode = Villa::CPCURRENT)
+    throw(Villa_error);
+  /**
+   * Delete the record where the cursor is.
+   * @throw Villa_error if an error occurs or no record corresponds to the cursor.
+   * @note After deletion, the cursor is moved to the next record if possible.
+   */
+  virtual void curout() throw(Villa_error);
   /**
    * Set the tuning parameters for performance.
    * @param lrecmax the max number of records in a leaf node of B+ tree.  If it is not more
@@ -462,7 +492,7 @@ public:
    * @return the last modified time the database.
    * @throw Villa_error if an error occurs.
    */
-  virtual int mtime() throw(Villa_error);
+  virtual time_t mtime() throw(Villa_error);
   /**
    * Begin the transaction.
    * @throw Villa_error if an error occurs.
