@@ -1,6 +1,6 @@
 #=================================================================================================
 # Ruby API of Villa, the advanced API of QDBM
-#                                                       Copyright (C) 2000-2005 Mikio Hirabayashi
+#                                                       Copyright (C) 2000-2006 Mikio Hirabayashi
 # This file is part of QDBM, Quick Database Manager.
 # QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
 # Lesser General Public License as published by the Free Software Foundation; either version
@@ -41,17 +41,19 @@ class Villa
   #----------------------------------------------------------------
   public
   ##
-  # villa = Villa::new(name, omode, bnum)
+  # villa = Villa::new(name, omode, cmode)
   # Constructor: Get a database handle.
   # `name' specifies the name of a database file.
   # `omode' specifies the connection mode: `Villa::OWRITER' as a writer, `Villa::OREADER' as a
   # reader.  If the mode is `Villa::OWRITER', the following may be added by bitwise or:
   # `Villa::OCREAT', which means it creates a new database if not exist, `Villa::OTRUNC', which
   # means it creates a new database regardless if one exists, `Villa::OZCOMP', which means
-  # leaves in the database are compressed.  Both of `Villa::OREADER' and `Villa::OWRITER' can
-  # be added to by bitwise or: `Villa::ONOLCK', which means it opens a database file without
-  # file locking, or `Villa::OLCKNB', which means locking is performed without blocking.  If it
-  # is omitted, `Villa::OREADER' is specified.
+  # leaves in the database are compressed with ZLIB, `Villa::OYCOMP', which means leaves in the
+  # database are compressed with LZO, `Villa::OXCOMP', which means leaves in the database are
+  # compressed with BZIP2.  Both of `Villa::OREADER' and `Villa::OWRITER' can be added to by
+  # bitwise or: `Villa::ONOLCK', which means it opens a database file without file locking, or
+  # `Villa::OLCKNB', which means locking is performed without blocking.  If it is omitted,
+  # `Villa::OREADER' is specified.
   # `cmode' specifies the comparing function: `Villa::CMPLEX' comparing keys in lexical order,
   # `Villa::CMPDEC' comparing keys as decimal strings, `Villa::CMPOBJ' comparing keys as
   # serialized objects implementing the method `<=>'.  The comparing function should be kept
@@ -66,13 +68,13 @@ class Villa
   # closed when the block is over.
   # While connecting as a writer, an exclusive lock is invoked to the database file.
   # While connecting as a reader, a shared lock is invoked to the database file.  The thread
-  # blocks until the lock is achieved.  `Villa::OZCOMP' is available only if QDBM was built
-  # with ZLIB enabled.  If `Villa::ONOLCK' is used, the application is responsible for exclusion
-  # control.
+  # blocks until the lock is achieved.  `Villa::OZCOMP', `Villa::OYCOMP', and `Villa::OXCOMP'
+  # are available only if QDBM was built each with ZLIB, LZO, and BZIP2 enabled.  If
+  # `Villa::ONOLCK' is used, the application is responsible for exclusion control.
   #
   #@ DEFINED IMPLICITLY
   ##
-  # villa = Villa::open(name, omode, cmp)
+  # villa = Villa::open(name, omode, cmode)
   # Constructor: An alias of `new'.
   #
   #@ DEFINED OUTSIDE
@@ -141,15 +143,16 @@ class Villa
   # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
   # `val' specifies a value.  Although it must be an instance of String, binary data is okey.
   # `dmode' specifies behavior when the key overlaps, by the following values: `Villa::DOVER',
-  # which means the specified value overwrites the existing one, `Villa::DKEEP', which means
-  # the existing value is kept, `Villa::DCAT', which means the specified value is concatenated
-  # at the end of the existing value, `Villa::DDUP', which means duplication of keys is allowed.
-  # If it is omitted `Villa::DOVER' is specified.
+  # which means the specified value overwrites the existing one, `Villa::DKEEP', which means the
+  # existing value is kept, `Villa::DCAT', which means the specified value is concatenated at the
+  # end of the existing value, `Villa::DDUP', which means duplication of keys is allowed and the
+  # specified value is added as the last one, `Villa::DDUPR', which means duplication of keys is
+  # allowed and the specified value is added as the first one.  If it is omitted, `Villa::DOVER'
+  # is specified.
   # The return value is always `true'.
   # An exception of `Villa::EANY' or its sub classes is thrown if an error occurs or replace
   # is cancelled.
-  # A duplicated record is stored at the tail of the records of the same key.  The cursor
-  # becomes unavailable due to updating database.
+  # The cursor becomes unavailable due to updating database.
   #
   def put(key, val, dmode = DOVER)
     if(@cmode == CMPOBJ)
@@ -314,7 +317,8 @@ class Villa
   # to the first record of the same key and that the cursor is set to the next substitute if
   # completely matching record does not exist, `Villa::JBACKWARD', which means that the cursor
   # is set to the last record of the same key and that the cursor is set to the previous
-  # substitute if completely matching record does not exist.
+  # substitute if completely matching record does not exist.  If it is omitted, `Villa::JFORWARD'
+  # is specified.
   # The return value is always `true'.
   # An exception of `Villa::EANY' or its sub classes is thrown if an error occurs or there is
   # no record corresponding the condition.
@@ -354,6 +358,36 @@ class Villa
     val
   end
   ##
+  # bool = villa.curput(val, cpmode);
+  # Method: Insert a record around the cursor.
+  # `val' specifies a value.  Although it must be an instance of String, binary data is okey.
+  # `cpmode' specifies detail adjustment: `Villa::CPCURRENT', which means that the value of the
+  # current record is overwritten, `Villa::CPBEFORE', which means that a new record is inserted
+  # before the current record, `Villa::CPAFTER', which means that a new record is inserted after
+  # the current record.  If it is omitted, `Villa::CPCURRENT' is specified.
+  # The return value is always `true'.
+  # An exception of `Villa::EANY' or its sub classes is thrown if an error occurs or no record
+  # corresponds to the cursor.
+  # After insertion, the cursor is moved to the inserted record.
+  #
+  def curput(val, cpmode = CPCURRENT)
+    if(@cmode == CMPOBJ)
+      val = Marshal.dump(val)
+    end
+    mod_curput(@index, val, cpmode)
+  end
+  ##
+  # bool = villa.curout();
+  # Method: Delete the record where the cursor is.
+  # The return value is always `true'.
+  # An exception of `Villa::EANY' or its sub classes is thrown if an error occurs or no record
+  # corresponds to the cursor.
+  # After deletion, the cursor is moved to the next record if possible.
+  #
+  def curout()
+    mod_curout(@index)
+  end
+  ##
   # bool = villa.settuning(lrecmax, nidxmax, lcnum, ncnum)
   # Method: Set the tuning parameters for performance.
   # `lrecmax' specifies the max number of records in a leaf node of B+ tree.  If it is not more
