@@ -29,12 +29,10 @@
 #define VL_DEFNIDXMAX  192               /* default number of indexes in each node */
 #define VL_DEFLCNUM    1024              /* default number of leaf cache */
 #define VL_DEFNCNUM    512               /* default number of node cache */
-#define VL_ALIGNRATIO  1.4               /* ratio between alignment and page size */
 #define VL_CACHEOUT    8                 /* number of pages in a process of cacheout */
 #define VL_INITBNUM    32749             /* initial bucket number */
-#define VL_INITALIGN   448               /* initial size of alignment */
-#define VL_OPTALIGN    -3                /* alignment setting when optimization */
-#define VL_FBPOOLSIZ   64                /* size of free block pool */
+#define VL_PAGEALIGN   -3                /* alignment for pages */
+#define VL_FBPOOLSIZ   128               /* size of free block pool */
 #define VL_PATHBUFSIZ  1024              /* size of a path buffer */
 #define VL_TMPFSUF     MYEXTSTR "vltmp"  /* suffix of a temporary file */
 #define VL_ROOTKEY     -1                /* key of the root key */
@@ -149,7 +147,8 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
     } else if(_qdbm_lzoencode && cmode == VL_OXCOMP){
       flags |= VL_FLISBZIP;
     }
-    if(!dpsetflags(depot, flags) || !dpsetfbpsiz(depot, VL_FBPOOLSIZ)){
+    if(!dpsetflags(depot, flags) || !dpsetalign(depot, VL_PAGEALIGN) ||
+       !dpsetfbpsiz(depot, VL_FBPOOLSIZ)){
       dpclose(depot);
       return NULL;
     }
@@ -173,8 +172,6 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
   villa->nodeidxmax = VL_DEFNIDXMAX;
   villa->leafcnum = VL_DEFLCNUM;
   villa->nodecnum = VL_DEFNCNUM;
-  villa->avglsiz = VL_INITALIGN;
-  villa->avgnsiz = VL_INITALIGN;
   villa->tran = FALSE;
   villa->rbroot = -1;
   villa->rblast = -1;
@@ -940,6 +937,7 @@ int vlsync(VILLA *villa){
   if(!vldpputnum(villa->depot, VL_NNUMKEY, villa->nnum)) err = TRUE;
   if(!vldpputnum(villa->depot, VL_RNUMKEY, villa->rnum)) err = TRUE;
   if(!dpsync(villa->depot)) err = TRUE;
+  if(!dpsetalign(villa->depot, VL_PAGEALIGN)) err = TRUE;
   return err ? FALSE : TRUE;
 }
 
@@ -958,7 +956,6 @@ int vloptimize(VILLA *villa){
   }
   err = FALSE;
   if(!vlsync(villa)) return FALSE;
-  if(!dpsetalign(villa->depot, VL_OPTALIGN)) err = TRUE;
   if(!dpoptimize(villa->depot, -1)) err = TRUE;
   return err ? FALSE : TRUE;
 }
@@ -1065,6 +1062,7 @@ int vltranbegin(VILLA *villa){
   if(!vldpputnum(villa->depot, VL_NNUMKEY, villa->nnum)) err = TRUE;
   if(!vldpputnum(villa->depot, VL_RNUMKEY, villa->rnum)) err = TRUE;
   if(!dpmemsync(villa->depot)) err = TRUE;
+  if(!dpsetalign(villa->depot, VL_PAGEALIGN)) err = TRUE;
   villa->tran = TRUE;
   villa->rbroot = villa->root;
   villa->rblast = villa->last;
@@ -1114,6 +1112,7 @@ int vltrancommit(VILLA *villa){
   if(!vldpputnum(villa->depot, VL_NNUMKEY, villa->nnum)) err = TRUE;
   if(!vldpputnum(villa->depot, VL_RNUMKEY, villa->rnum)) err = TRUE;
   if(!dpmemsync(villa->depot)) err = TRUE;
+  if(!dpsetalign(villa->depot, VL_PAGEALIGN)) err = TRUE;
   villa->tran = FALSE;
   villa->rbroot = -1;
   villa->rblast = -1;
@@ -1682,9 +1681,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
       dpecodeset(DP_EMISC, __FILE__, __LINE__);
       return FALSE;
     }
-    villa->avglsiz = (villa->avglsiz * 9 + zsiz) / 10;
-    if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
-       !dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
+    if(!dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
       cbdatumclose(buf);
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return FALSE;
@@ -1696,9 +1693,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
       dpecodeset(DP_EMISC, __FILE__, __LINE__);
       return FALSE;
     }
-    villa->avglsiz = (villa->avglsiz * 9 + zsiz) / 10;
-    if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
-       !dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
+    if(!dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
       cbdatumclose(buf);
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return FALSE;
@@ -1710,18 +1705,14 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
       dpecodeset(DP_EMISC, __FILE__, __LINE__);
       return FALSE;
     }
-    villa->avglsiz = (villa->avglsiz * 9 + zsiz) / 10;
-    if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
-       !dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
+    if(!dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
       cbdatumclose(buf);
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return FALSE;
     }
     free(zbuf);
   } else {
-    villa->avglsiz = (villa->avglsiz * 9 + CB_DATUMSIZE(buf)) / 10;
-    if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
-       !dpput(villa->depot, (char *)&(leaf->id), sizeof(int),
+    if(!dpput(villa->depot, (char *)&(leaf->id), sizeof(int),
               CB_DATUMPTR(buf), CB_DATUMSIZE(buf), DP_DOVER)){
       cbdatumclose(buf);
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
@@ -2027,9 +2018,7 @@ static int vlnodesave(VILLA *villa, VLNODE *node){
     cbdatumcat(buf, vnumbuf, vnumsiz);
     cbdatumcat(buf, CB_DATUMPTR(idxp->key), ksiz);
   }
-  villa->avgnsiz = (villa->avgnsiz * 9 + CB_DATUMSIZE(buf)) / 10;
-  if(!dpsetalign(villa->depot, (int)(villa->avgnsiz * VL_ALIGNRATIO)) ||
-     !dpput(villa->depot, (char *)&(node->id), sizeof(int),
+  if(!dpput(villa->depot, (char *)&(node->id), sizeof(int),
             CB_DATUMPTR(buf), CB_DATUMSIZE(buf), DP_DOVER)){
     cbdatumclose(buf);
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
