@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Utility for debugging Villa and its applications
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -57,7 +57,7 @@ int runimportdb(int argc, char **argv);
 void pdperror(const char *name);
 void printobj(const char *obj, int size);
 void printobjhex(const char *obj, int size);
-int docreate(const char *name, int zc);
+int docreate(const char *name, int cmode);
 int doput(const char *name, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
           int dmode, VLCFUNC cmp);
 int doout(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb);
@@ -106,7 +106,7 @@ int main(int argc, char **argv){
   } else if(!strcmp(argv[1], "version") || !strcmp(argv[1], "--version")){
     printf("Powered by QDBM version %s on %s%s\n",
            dpversion, dpsysname, dpisreentrant ? " (reentrant)" : "");
-    printf("Copyright (c) 2000-2005 Mikio Hirabayashi\n");
+    printf("Copyright (c) 2000-2006 Mikio Hirabayashi\n");
     rv = 0;
   } else {
     usage();
@@ -120,7 +120,7 @@ void usage(void){
   fprintf(stderr, "%s: administration utility for Villa\n", progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s create [-zc] name\n", progname);
+  fprintf(stderr, "  %s create [-cz|-cy|-cx] name\n", progname);
   fprintf(stderr, "  %s put [-kx|-ki] [-vx|-vi|-vf] [-keep|-cat|-dup] name key val\n", progname);
   fprintf(stderr, "  %s out [-l] [-kx|-ki] name key\n", progname);
   fprintf(stderr, "  %s get [-l] [-kx|-ki] [-ox] [-n] name key\n", progname);
@@ -175,13 +175,17 @@ char *dectoiobj(const char *str, int *sp){
 /* parse arguments of create command */
 int runcreate(int argc, char **argv){
   char *name;
-  int i, zc, rv;
+  int i, cmode, rv;
   name = NULL;
-  zc = FALSE;
+  cmode = 0;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-zc")){
-        zc = TRUE;
+      if(!strcmp(argv[i], "-cz")){
+        cmode |= VL_OZCOMP;
+      } else if(!strcmp(argv[i], "-cy")){
+        cmode |= VL_OYCOMP;
+      } else if(!strcmp(argv[i], "-cx")){
+        cmode |= VL_OXCOMP;
       } else {
         usage();
       }
@@ -192,7 +196,7 @@ int runcreate(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = docreate(name, zc);
+  rv = docreate(name, cmode);
   return rv;
 }
 
@@ -625,11 +629,10 @@ void printobjhex(const char *obj, int size){
 
 
 /* perform create command */
-int docreate(const char *name, int zc){
+int docreate(const char *name, int cmode){
   VILLA *villa;
   int omode;
-  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC;
-  if(zc) omode |= VL_OZCOMP;
+  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC | cmode;
   if(!(villa = vlopen(name, omode, VL_CMPLEX))){
     pdperror(name);
     return 1;
@@ -879,7 +882,7 @@ int doinform(const char *name){
   printf("non-leaf nodes: %d\n", vlnnum(villa));
   printf("records: %d\n", vlrnum(villa));
   printf("inode number: %d\n", vlinode(villa));
-  printf("modified time: %d\n", vlmtime(villa));
+  printf("modified time: %.0f\n", (double)vlmtime(villa));
   if(!vlclose(villa)){
     pdperror(name);
     return 1;
